: PMixp
: localized mixed permeability current
: number of open channels is driven by an external value
: that is communicated by a POINTER

NEURON {
  POINT_PROCESS PMixp
  POINTER num
  USEION na READ ena, nai, nao WRITE ina
  USEION k READ ek, ki, ko WRITE ik
:  RANGE gna1, gk1
:  RANGE gna, gk, ina, ik
  RANGE pna1, pk1
  RANGE pna, pk, ina, ik
}

UNITS {
:  (uS) = (microsiemens)
:  (pS) = (picosiemens)
  (mV) = (millivolt)
  (mM) = (milli/liter)
  (um) = (micron)
  FARADAY = (faraday) (coulomb)
  R = (k-mole) (joule/degC)
}

PARAMETER {
:  gna1 = 0 (pS)  < 0, 1e9 > : single channel conductances
:  gk1 = 0 (pS)  < 0, 1e9 >
  pna1 = 5e-3 (cm/s)  < 0, 1e9 > : single channel permeabilities
  pk1 = 10e-3 (cm/s)  < 0, 1e9 >
}

ASSIGNED {
  v  (millivolt)
  celsius (degC)
  num (1) : how many are open
  ena  (millivolt)
  nai (mM)
  nao (mM)
:  gna  (uS)
  pna  (cm/s)
  ina  (nanoamp)
  ek  (millivolt)
  ki (mM)
  ko (mM)
:  gk  (uS)
  pk  (cm/s)
  ik  (nanoamp)
  area (um2)
}

BREAKPOINT {
:  gna = (1e-6)*num*gna1
:  ina = gna*(v - ena)
  pna = num*pna1
  : pna*ghk(v, nai, nao, 1) returns current in mA/cm2
  : however this mechanism is a point process
  : so we need to multiply by (compartment area)/100
  : to convert current density to net current in nA
  ina = (0.01)*area*pna*ghk(v, nai, nao, 1)
:  gk = (1e-6)*num*gk1
:  ik = gk*(v - ek)
  pk = num*pk1
  ik = (0.01)*area*pk*ghk(v, ki, ko, 1)
}

: argument z is valence, e.g. +1, +2, -1
FUNCTION ghk(v(mV), ci(mM), co(mM), z(1)) (0.001 coul/cm3) {
  LOCAL tmp, eci, eco
  tmp = (1e-3)*z*FARADAY*v/(R*(celsius+273.15))
  eco = co*efun(tmp)
  eci = ci*efun(-tmp)
  ghk = (0.001)*FARADAY*(eci - eco)
}

FUNCTION efun(z) {
  if (fabs(z) < 1e-4) {
    efun = 1 - z/2
  }else{
    efun = z/(exp(z) - 1)
  }
}

